package cn.fw.morax.common.utils;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 线程池 工具类 (采用枚举类型的单例模式)
 *
 * @author kurisu
 * @date 2022-04-15 15:28
 */
public class ThreadPoolUtil {
    /**
     * 获取实例
     */
    public static ThreadPoolUtil getInstance() {
        return innerEnum.INSTANCE.threadPoolUtil();
    }

    /**
     * 获取线程池
     */
    public ThreadPoolExecutor getThreadPool() {
        return innerEnum.INSTANCE.getThreadPool();
    }

    private enum innerEnum {
        /**
         * 线程池枚举
         */
        INSTANCE;
        private final static int CORE_SIZE = 10;
        private final static int MAXIMUM_SIZE = 100;
        private final static int MAX_QUEUE_SIZE = 2000;
        private final static long KEEP_ALIVE_TIME = 30L;
        private final ThreadPoolExecutor executor;
        private final ThreadPoolUtil threadPoolUtil;

        innerEnum() {
            LinkedBlockingQueue<Runnable> queue = new LinkedBlockingQueue<>(MAX_QUEUE_SIZE);
            executor = new ThreadPoolExecutor(CORE_SIZE, MAXIMUM_SIZE,
                    KEEP_ALIVE_TIME, TimeUnit.SECONDS, queue,
                    new ThreadPoolExecutor.AbortPolicy()
            );
            threadPoolUtil = new ThreadPoolUtil();
        }

        private ThreadPoolExecutor getThreadPool() {
            return executor;
        }

        private ThreadPoolUtil threadPoolUtil() {
            return threadPoolUtil;
        }
    }
}