package cn.fw.morax.dao.salary;

import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.dto.query.SalaryGroupRepeatQueryDTO;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author : kurisu
 * @className : SalaryGroupDao
 * @description : 薪酬组配置
 * @date: 2022-04-06 14:51
 */
@Repository
public interface SalaryGroupDao extends BaseMapper<SalaryGroup> {

    /**
     * 查询重复的薪酬组配置
     *
     * @param dto
     * @return
     */
    List<SalaryGroup> queryRepeatSalaryGroup(SalaryGroupRepeatQueryDTO dto);

    /**
     * 查询指定日期有效的薪酬组ids
     *
     * @param day
     * @return
     */
    Set<Long> querySalaryGroupIdByDay(@Param("day") Date day);

    /**
     * 根据指标编码和岗位码查询集团所有薪酬组id
     *
     * @param salaryProjectCode 指标编码
     * @param postId        岗位Id
     * @param groupId       集团id
     * @param dataDate      数据日期
     * @return List
     */
    List<Long> querySalaryGroupIdsByCode(@Param("salaryProjectCode") final String salaryProjectCode,
                                          @Param("postId") final Long postId,
                                          @Param("groupId") final Long groupId,
                                          @Param("dataDate") final Date dataDate
    );

    /**
     * 获取所有生效中薪酬组数据
     *
     * @param day
     * @return
     */
    List<SalaryGroup> getAllEffectGroups(Date day);

    /**
     * 门店岗位重复的薪酬组
     *
     * @param postId
     * @param shopIds
     * @param preId
     * @param beginTime
     * @return
     */
    List<SalaryGroup> getApprovingConfigs(@Param("postId") Long postId,
                                          @Param("shopIds") List<Long> shopIds,
                                          @Param("preId") Long preId,
                                          @Param("beginTime") LocalDate beginTime);
}
