package cn.fw.morax.domain.db;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.morax.domain.enums.SalaryPayrollStatusEnum;
import lombok.*;

import java.math.BigDecimal;
import java.time.YearMonth;

/**
 * 薪资发放记录
 *
 * @author : kurisu
 * @version : 1.0
 * @className : PayrollRecord
 * @description : 薪资发放记录
 * @date : 2022-05-06 18:16
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class PayrollRecord extends BaseAuditableTimeEntity<PayrollRecord, Long> {
    /**
     * 用户id
     */
    private Long userId;
    /**
     * 用户名称
     */
    private String userName;
    /**
     * 门店id
     */
    private Long shopId;
    /**
     * 社保公积金门店id
     */
    private Long fundShopId;
    /**
     * 商家id
     */
    private Long dealerId;
    /**
     * 业务id
     */
    private String bizId;
    /**
     * 是否是付款
     * 否的情况这是需要收取员工的金额
     */
    private Boolean payment;
    /**
     * 是否是工资
     * 否的情况则是及时发放的奖励或者扣款
     */
    private Boolean salary;
    /**
     * 金额
     */
    private BigDecimal amount;
    /**
     * 公帐发放
     */
    private Boolean corporate;
    /**
     * 发放状态
     */
    private SalaryPayrollStatusEnum payrollStatus;
    /**
     * 财务token
     */
    private String payrollToken;
    /**
     * 月度
     */
    private YearMonth monthly;
    /**
     * 集团id
     */
    private Long groupId;
}
