package cn.fw.morax.domain.dto;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.Indicators;
import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import cn.fw.morax.domain.enums.KpiTargetTypeEnum;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 指标库
 */
@Data
public class IndicatorsDTO {

    /**
     * 指标名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String indicatorName;

    /**
     * 归属系统id
     */
    @NotNull(message = "归属系统不能为空")
    private Long sysId;

    /**
     * 归属系统名称
     */
    @NotNull(message = "归属系统名称不能为空")
    private String sysName;

    /**
     * 适用角色类型;1: 全部角色 2:全部管理角色 3:自定义
     */
    @NotNull(message = "适用角色类型不能为空")
    private ApplicableRoleEnum roleType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    @NotNull(message = "绩效目标值类型不能为空")
    private KpiTargetTypeEnum targetType;

    /**
     * 适用角色编码
     */
//    @NotNull(message = "适用角色编码不能为空")
//    @Size(min = 1, message = "适用角色编码不能为空")
    private List<String> roleCodes;

    /**
     * 适用角色名称
     */
//    @NotNull(message = "适用角色名称不能为空")
//    @Size(min = 1, message = "适用角色名称不能为空")
    private List<String> roleNames;

    /**
     * 转换为db对象
     * @return
     */
    public Indicators convertToPo(){
        Indicators indicators = Indicators.builder()
                .indicatorName(this.getIndicatorName())
                .enable(Boolean.FALSE)
                .roleType(this.getRoleType())
                .targetType(this.targetType)
                .sysId(this.getSysId())
                .sysName(this.getSysName())
                .build();
        if (ApplicableRoleEnum.PORTION.equals(this.getRoleType())) {
            if (PublicUtil.isNotEmpty(this.getRoleCodes())) {
                indicators.setRoleCodes(this.getRoleCodes());
            }
            if (PublicUtil.isNotEmpty(this.getRoleNames())) {
                indicators.setRoleNames(this.getRoleNames());
            }
        }
        return indicators;
    }

}