package cn.fw.morax.domain.dto;

import cn.fw.morax.domain.enums.KpiTargetTypeEnum;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组指标项
 */
@Data
public class KpiGroupIndicatorDTO {

    /**
     * 指标库id
     */
    @NotNull(message = "绩效组指标项不能为空")
    private Long indicatorId;

    /**
     * 指标库名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String indicatorName;

    /**
     * 指标库名称
     */
    @NotBlank(message = "指标编码不能为空")
    @Length(max = 32, message = "指标编码超出最大长度限制")
    private String indicatorCode;

    /**
     * 基础绩效分
     */
    @NotNull(message = "基础绩效分不能为空")
    private Integer baseScore;

    /**
     * 得分方式;1: 阶梯得分 2:普通得分
     */
    @NotNull(message = "得分方式不能为空")
    private ScoreWayEnum scoreWay;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    @NotNull(message = "绩效目标值类型不能为空")
    private KpiTargetTypeEnum targetType;
    /**
     * 绩效目标值
     */
    private BigDecimal targetValue;

    /**
     * 得分阶梯
     */
    @Valid
    private List<KpiGroupIndicatorLaddersDTO> indicatorLadders;

}