package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : EffectMonthEnum
 * @description : 生效月份
 * @date: 2022-04-06 14:18
 */
public enum EffectMonthEnum implements IEnum<Integer> {
    /**
     * 生效月份类型;1:当月 2:次月
     */
    CURRENT_MONTH(1, "当月"),
    NEXT_MONTH(2, "次月"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    EffectMonthEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static EffectMonthEnum ofValue(final Integer value) {
        for (final EffectMonthEnum _enum : EffectMonthEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final EffectMonthEnum _enum : EffectMonthEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
