package cn.fw.morax.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * 工资发放状态枚举
 *
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryPayrollStatusEnum
 * @description : 工资发放状态枚举
 * @date : 2022-05-06 18:24
 */
public enum SalaryPayrollStatusEnum implements IEnum<Integer> {
    /**
     * 发放状态;1:待发放 2:已推财务 3:已完成
     */
    WAITING(1, "待发放"),
    SEND(2, "已推财务"),
    COMPLETE(3, "已完成"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    SalaryPayrollStatusEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SalaryPayrollStatusEnum ofValue(final Integer value) {
        for (final SalaryPayrollStatusEnum _enum : SalaryPayrollStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SalaryPayrollStatusEnum _enum : SalaryPayrollStatusEnum.values()) {
            if (_enum.value.equals(value)) {
                return _enum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
