package cn.fw.morax.domain.vo.kpi;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.domain.enums.StarEvaluationEnum;
import lombok.Data;

import java.math.BigDecimal;
import java.time.YearMonth;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-6
 * @desc : 绩效组配置
 */
@Data
public class KpiGroupVO {
    /**
     * 主键id
     */
    private Long id;
    /**
     * 绩效组编码
     */
    private String kgc;
    /**
     * 名称
     */
    private String name;
    /**
     * 岗位id
     */
    private Long postId;
    /**
     * 岗位名称
     */
    private String postName;
    /**
     * 适用门店ids
     */
    private List<Long> shopIds;
    /**
     * 适用门店名称
     */
    private List<String> shopNames;
    /**
     * 生效时间
     */
    private YearMonth beginTime;
//    /**
//     * 失效时间
//     */
//    private YearMonth overTime;
    /**
     * 调整原因
     */
    private String reason;
    /**
     * 附件;fid逗号隔开
     */
    private List<String> attachment;
    /**
     * 状态;1:审批中 2:待生效 3:生效中 4:已失效
     */
    private SettingStatusEnum status;
    /**
     * 包含指标数量
     */
    private Integer indicatorNum;
    /**
     * 星级评定方式;1:绩效得分率 2:人员百分比
     */
    private StarEvaluationEnum starEvaluationType;
    /**
     * 集团id
     */
    private Long groupId;

    /**
     * 撤销D级得分率
     */
    private BigDecimal revokedScoreRatio;

    /**
     * 绩效组指标项
     */
    private List<KpiGroupIndicatorVO> indicators;

    /**
     * 星级评定阶梯
     */
    private List<KpiStarLaddersVO> starLadders;


    /**
     * 装换方法
     * @param kpiGroup
     */
    public static KpiGroupVO convertKpiGroup(KpiGroup kpiGroup) {
        KpiGroupVO kpiGroupVo = new KpiGroupVO();
        kpiGroupVo.setId(kpiGroup.getId());
        kpiGroupVo.setPostId(kpiGroup.getPostId());
        kpiGroupVo.setName(kpiGroup.getName());
        kpiGroupVo.setPostName(kpiGroup.getPostName());
        kpiGroupVo.setShopIds(kpiGroup.getShopIds());
        kpiGroupVo.setShopNames(kpiGroup.getShopNames());
//        kpiGroupVo.setPreId(PublicUtil.isNotEmpty(kpiGroup.getPreId()) ? kpiGroup.getPreId() : null);
        kpiGroupVo.setBeginTime(YearMonth.from(kpiGroup.getBeginTime()));
//        kpiGroupVo.setOverTime(kpiGroup.getOverTime());
        kpiGroupVo.setReason(kpiGroup.getReason());
        kpiGroupVo.setAttachment(kpiGroup.getAttachment());
        kpiGroupVo.setStatus(kpiGroup.getStatus());
        kpiGroupVo.setIndicatorNum(kpiGroup.getIndicatorNum());
        kpiGroupVo.setStarEvaluationType(kpiGroup.getStarEvaluationType());
        kpiGroupVo.setGroupId(kpiGroup.getGroupId());
        if (PublicUtil.isNotEmpty(kpiGroup.getRevokedScoreRatio())) {
            kpiGroupVo.setRevokedScoreRatio(kpiGroup.getRevokedScoreRatio().multiply(Constant.ONE_HUNDRED));
        }
        return kpiGroupVo;
    }

}