package cn.fw.morax.sdk.dto.kpi;

import lombok.*;

import javax.validation.constraints.*;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : UserIndicatorReq
 * @description : 人员绩效指标上报
 * @date : 2022-04-13 16:45
 */
@Data
@ToString
@EqualsAndHashCode
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class UserIndicatorReq {
    /**
     * 指标编码
     */
    @NotBlank(message = "指标编码不能为空")
    private String indicatorCode;
    /**
     * 岗位id
     */
    @NotNull(message = "岗位ID不能为空")
    private Long postId;
    /**
     * 人员id
     */
    @NotNull(message = "人员id不能为空")
    private Long userId;
    /**
     * 指标对应值
     * (如果是百分比的情况则使用0.x)
     * eg. 56.5% 则对应值为 0.565
     */
    @DecimalMin(value = "0", message = "指标值必须大于等于0")
    private BigDecimal value;
    /**
     * 门店id
     */
    @NotNull(message = "门店id不能为空")
    private Long shopId;
    /**
     * 数据日期
     */
    @NotNull(message = "数据日期不能为空")
    @Past(message = "数据日期必须为过去的时间")
    private Date dataDate;
    /**
     * 唯一标识
     */
    @NotNull(message = "标识码不能为空")
    @Min(value = 1, message = "标识码不合法")
    private Long uid;
}

