package cn.fw.morax.sdk.dto.salary;

import cn.fw.common.validator.EnumValue;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.util.Date;
import java.util.List;

/**
 * @author : kurisu
 * @className : SalaryDataReportReq
 * @description : 薪酬数据上报类
 * @date : 2022-04-07 15:29
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class SalaryDataReportReq {
    /**
     * 薪酬项编码
     */
    @NotBlank(message = "薪酬项编码不能为空")
    private String salaryProjectCode;
    /**
     * 用户id
     */
    @NotNull(message = "用户id不能为空")
    private Long userId;
    /**
     * 上报日期
     */
    @NotNull(message = "上报日期不能为空")
    @Past(message = "上报日期必须为过去的时间")
    private Date dataDate;
    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;
    /**
     * 绩效值类型
     *
     * @see SalaryValueEnum
     */
    @NotNull(message = "绩效值类型不能为空")
    @EnumValue(enumClass = SalaryValueEnum.class, valueMethod = "getValue", message = "绩效值枚举值不正确")
    private Integer valueType;
    /**
     * 薪酬项对应值
     */
    @NotEmpty(message = "薪酬项对应值不能为空")
    @Valid
    private List<SalaryValueDTO> salaryValue;
}
