package cn.fw.morax.server.controller.erp;

import cn.fw.approval.sdk.mq.ApprovalResult;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.bo.kpi.KpiGroupCalculableBO;
import cn.fw.morax.domain.dto.query.KpiGroupStaffQueryDTO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.server.consumer.FlowConsumer;
import cn.fw.morax.server.task.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : DebugController
 * @description : 绩效通用调试控制器
 * @date : 2022-04-11 10:45
 * @ignore
 */
@Slf4j
@RestController
@Authorization(AuthType.NONE)
@RequiredArgsConstructor
@Validated
@RequestMapping("/debug/task/kpi")
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiTaskController {
    private final SalaryPoolFundShopTask salaryPoolFundShopTask;
    private final KpiStarSpecialRuleTask kpiStarSpecialRuleTask;
    private final KpiGroupUserTask kpiGroupUserTask;
    private final SalaryGroupTask salaryGroupTask;
    private final KpiGroupTask kpiGroupTask;
    private final EhrRpcService ehrRpcService;
    private final FlowConsumer flowConsumer;
    private final KpiCalcTask kpiCalcTask;


    @GetMapping("/current-month/kpi-config")
    @ControllerMethod("当月绩效组定时器")
    public Message<Void> processCurMonthEffectKpis() {
        kpiGroupTask.processCurMonthEffectKpis();
        return success();
    }

    @GetMapping("/current-month/salary-config")
    @ControllerMethod("当月薪酬组定时器")
    public Message<Void> processCurMonthEffectSalaryGroup() {
        salaryGroupTask.processCurMonthEffectSalaryGroup();
        return success();
    }

    @GetMapping("/kpi-users")
    @ControllerMethod("当天绩效组人员定时器")
    public Message<Void> processKpiUser() {
        kpiGroupUserTask.processKpiUser();
        return success();
    }

    @GetMapping("/salary-user")
    @ControllerMethod("当天薪酬组人员定时器")
    public Message<Void> salaryGroupUser() {
        salaryGroupTask.asyncSalaryGroupUser();
        return success();
    }

    @GetMapping("/calculate-kpi")
    @ControllerMethod("计算绩效得分")
    public Message<Void> calculateKpi() {
        kpiCalcTask.calculateKpi();
        return success();
    }

    @GetMapping("/calculate-kpi-rate")
    @ControllerMethod("计算绩效得分率")
    public Message<Void> calcuScoringRate() {
        kpiCalcTask.calcuScoringRate();
        return success();
    }

    @GetMapping("/calculate-star")
    @ControllerMethod("计算绩效星级")
    public Message<Void> calcuStarLevel() {
        kpiCalcTask.calcuStarLevel();
        return success();
    }

    @GetMapping("/cache/kpi-group")
    @ControllerMethod("需要计算绩效得分的绩效组写入缓存")
    public Message<Void> cacheCalculableKpiGroup() {
        kpiCalcTask.cacheCalculableKpiGroup();
        return success();
    }

    @GetMapping("/cache/kpi-rank")
    @ControllerMethod("需要计算绩效排名的绩效组写入缓存")
    public Message<Void> cacheCalculableKpiGroupRank() {
        kpiCalcTask.cacheCalculableKpiGroupRank();
        return success();
    }

    /**
     * 实时查询绩效组人员
     * @param dto
     * @return
     */
    @PostMapping("/post-shop/staffs")
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<PerformanceStaffDTO>> queryShopPostRealTimeStaffs(@Validated @RequestBody KpiGroupStaffQueryDTO dto) {
        return success(ehrRpcService.queryKpiStaff(dto.getPostId(), dto.getShopIds()));
    }

    /**
     * 审批消息
     * @param result
     * @return
     */
    @PostMapping("/flow")
    @ControllerMethod("审批消息")
    public Message<Void> approvalResult(@Validated @RequestBody ApprovalResult result) {
        flowConsumer.onMessage(result);
        return success();
    }

    /**
     * 社保公积金门店赋值
     *
     * @return
     */
    @PostMapping("/fund")
    @ControllerMethod("社保公积金门店赋值")
    public Message<Void> dealSalaryPoolFundShop() {
        salaryPoolFundShopTask.dealSalaryPoolFundShop();
        return success();
    }

    /**
     * 发送业务上报mq通知
     *
     * @return
     */
    @GetMapping("/kpi-mp")
    @ControllerMethod("发送业务上报mq通知")
    public Message<Void> sendKpiNotice() {
        kpiCalcTask.sendNotice();
        return success();
    }

    /**
     * 几月开始计算绩效的星级调整
     *
     * @return
     */
    @GetMapping("/kpi-adjustment")
    @ControllerMethod("几月开始计算绩效的星级调整")
    public Message<Void> processKpiStarSpecialRule() {
        kpiStarSpecialRuleTask.processKpiStarSpecialRule();
        return success();
    }

}
