package cn.fw.morax.server.controller.erp;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.PaySalaryEvent;
import cn.fw.morax.common.utils.EventBusUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.bo.salary.SalaryGroupCalculableBO;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.domain.db.salary.SalaryPoolDetail;
import cn.fw.morax.domain.db.salary.SalaryProjectHitLog;
import cn.fw.morax.domain.dto.query.SalaryGroupRepeatQueryDTO;
import cn.fw.morax.domain.dto.query.StaffSalaryQueryDTO;
import cn.fw.morax.domain.enums.SalaryTypeEnum;
import cn.fw.morax.domain.enums.StarLevelEnum;
import cn.fw.morax.domain.vo.salary.SalaryGroupProjectVO;
import cn.fw.morax.domain.vo.salary.SalaryGroupVO;
import cn.fw.morax.domain.vo.salary.StaffSalaryDetailVO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.server.task.PayoffTask;
import cn.fw.morax.service.biz.salary.*;
import cn.fw.morax.service.data.salary.SalaryGroupService;
import cn.fw.morax.service.data.salary.SalaryPoolDetailService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import cn.fw.morax.service.data.salary.SalaryProjectHitLogService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryDebugController
 * @description : 薪酬通用调试控制器
 * @date : 2022-04-11 10:45
 * @ignore
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.NONE)
@Validated
@RequestMapping("/debug/salary")
public class SalaryDebugController {
    private final SalaryGroupUserBizService salaryGroupUserBizService;
    private final SalaryGroupBizService salaryGroupBizService;
    private final SalaryPoolBizService salaryPoolBizService;
    private final SalaryPoolService salaryPoolService;
    private final SalaryPoolDetailService salaryPoolDetailService;
    private final SalaryProjectHitLogService salaryProjectHitLogService;
    private final SalaryGroupService salaryGroupService;
    private final SalaryCalcService salaryCalcService;
    private final PayoffBizService payoffBizService;
    private final SalaryGroupDataService salaryGroupDataService;
    private final SalaryConfirmBizService salaryConfirmBizService;
    private final ErpRpcService erpRpcService;


    @PutMapping("/calc/user-group-salary")
    @ControllerMethod("计算薪酬组工资")
    public Message<Void> calcUserGroupSalary(@RequestParam("salaryGroupId") Long salaryGroupId,
                                             @RequestParam("calcTotal") boolean calcTotal,
                                             @RequestParam("calcDate") LocalDate calcDate) {
        salaryCalcService.calculateSalary(new SalaryGroupCalculableBO(salaryGroupId, calcDate, calcTotal));
        return success();
    }

    @PutMapping("/calc/user-salary")
    @ControllerMethod("计算用户工资")
    public Message<Void> calcUserSalary(@RequestParam("salaryGroupUserId") Long salaryGroupUserId,
                                        @RequestParam("calcTotal") boolean calcTotal) {
        salaryCalcService.calcUserSalary(salaryGroupUserId, calcTotal);
        return success();
    }

    @PutMapping("/group/async-user")
    @ControllerMethod("同步薪酬组人员")
    public Message<Void> calcUserSalary(@RequestParam("salaryGroupId") Long salaryGroupId) {
        salaryGroupUserBizService.asyncSalaryGroupUser(salaryGroupId);
        return success();
    }

    /**
     * 获取员工薪酬
     *
     * @return 薪酬池选项
     * @ignoreParams user
     */
    @GetMapping("/staffs")
    @ControllerMethod("获取员工薪酬")
    public Message<List<StaffSalaryDetailVO>> getStaffsSalary(Long userId, @RequestParam(required = false) YearMonth yearMonth) {
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now().minusMonths(1);
        }
        List<Long> shopIds = erpRpcService.queryUserRoleShopIds(userId, Constant.ROLE_CODE_WAGE_PAYMENT);
        List<StaffSalaryDetailVO> staffSalaryDetailVOS = salaryPoolBizService.getStaffsSalary(null, shopIds, yearMonth);
        return success(staffSalaryDetailVOS);
    }

    /**
     * 查询重复薪酬组数据
     *
     * @return 薪酬组
     * @ignoreParams user
     */
    @GetMapping("/repeat/group")
    @IgnoreUserToken
    @ControllerMethod("查询重复薪酬组数据")
    public Message<List<SalaryGroupVO>> queryRepeatSalaryGroup(@Validated SalaryGroupRepeatQueryDTO dto) {
        List<SalaryGroup> salaryGroups = salaryGroupService.queryRepeatSalaryGroup(dto);
        if (salaryGroups.isEmpty()) {
            return success();
        }
        List<SalaryGroupVO> salaryGroupVOS = PublicUtil.copyList(salaryGroups, SalaryGroupVO.class);
        return success(salaryGroupVOS);
    }

    /**
     * 薪酬确认
     *
     * @return
     * @ignoreParams user
     */
    @GetMapping("/confirm")
    @IgnoreUserToken
    @ControllerMethod("薪酬确认")
    public Message<List<SalaryGroupVO>> queryRepeatSalaryGroup() {
        salaryConfirmBizService.createSalaryConfirmTodo(2L);
        return success();
    }

    /**
     * 薪酬确认待办推送
     *
     * @return
     */
    @GetMapping("/confirm/push")
    @IgnoreUserToken
    @ControllerMethod("薪酬确认待办推送")
    public Message<List<SalaryGroupVO>> pushConfirm() {
        salaryConfirmBizService.pushConfirm();
        return success();
    }

    /**
     * 支付薪酬推送待办
     *
     * @return
     * @ignoreParams user
     */
    @GetMapping("/pay/push")
    @IgnoreUserToken
    @ControllerMethod("支付薪酬推送待办")
    public Message<List<SalaryGroupVO>> pushConfirm(Long id) {
        SalaryPool pool = salaryPoolService.getById(id);
        EventBusUtil.asyncPost(PaySalaryEvent.builder()
                .monthly(pool.getMonthly())
                .groupId(pool.getGroupId())
                .shopId(pool.getShopId())
                .shopName(pool.getShopName())
                .build());
        return success();
    }

//    @GetMapping("/init-sgc")
//    @ControllerMethod("初始化薪酬组编码")
//    public Message<Void> initSgc() {
//        salaryGroupDataService.initSgc();
//        return success();
//    }

    /**
     * 生成薪酬池相关数据
     *
     * @return 薪酬组
     * @ignoreParams user
     */
    @GetMapping("/generate/salary-data")
    @ControllerMethod("生成薪酬池相关数据")
    public Message<Void> generateSalaryData(Long userId, Long groupId, String userName, Long salaryGroupId,
                                            @RequestParam(required = false) YearMonth yearMonth) {

        SalaryGroupVO salaryGroup = salaryGroupBizService.getSalaryGroupDetail(salaryGroupId);
        LocalDate localDate;
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now();
            localDate = LocalDate.now();
        } else {
            localDate = yearMonth.atEndOfMonth();
        }

        SalaryPool salaryPool = salaryPoolService.getOne(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getSalaryGroupId, salaryGroupId)
                .eq(SalaryPool::getUserId, userId)
                .eq(SalaryPool::getMonthly, yearMonth));
        if (PublicUtil.isEmpty(salaryPool)) {
            salaryPool = new SalaryPool();
            salaryPool.setSalaryGroupId(salaryGroupId);
            salaryPool.setUserId(userId);
            salaryPool.setUserName(userName);
            salaryPool.setPostId(salaryGroup.getPostId());
            salaryPool.setPostName(salaryGroup.getPostName());
            salaryPool.setShopId(salaryGroup.getShopIds().get(0));
            salaryPool.setShopName(salaryGroup.getShopNames().get(0));
            salaryPool.setReward(new BigDecimal(20));
            salaryPool.setStarLevel(StarLevelEnum.D);
            salaryPool.setPaid(Boolean.FALSE);
            salaryPool.setMonthly(yearMonth);
            salaryPool.setRegular(Boolean.FALSE);
            salaryPool.setGroupId(groupId);
            salaryPoolService.save(salaryPool);
        }
        Long salaryPoolId = salaryPool.getId();

        BigDecimal wages = BigDecimal.ZERO;
        List<SalaryPoolDetail> salaryPoolDetails = Lists.newArrayList();
        List<SalaryProjectHitLog> hitLogs = Lists.newArrayList();
        for (SalaryGroupProjectVO groupProjectVO : salaryGroup.getProjects()) {
            SalaryPoolDetail poolDetail = new SalaryPoolDetail();
            poolDetail.setSalaryPoolId(salaryPoolId);
            poolDetail.setSalaryProjectId(groupProjectVO.getSalaryProjectId());
            poolDetail.setSalaryProjectName(groupProjectVO.getSalaryProjectName());
            poolDetail.setType(SalaryTypeEnum.ROYALTIES);
            poolDetail.setSalaryAmount(new BigDecimal(1000));
            poolDetail.setProcessedAmount(new BigDecimal(0));
            poolDetail.setActualSalaryAmount(new BigDecimal(1000));
            poolDetail.setSalaryDate(localDate);
            poolDetail.setGroupId(groupId);
            wages = wages.add(poolDetail.getActualSalaryAmount());
            salaryPoolDetails.add(poolDetail);

            if (PublicUtil.isNotEmpty(groupProjectVO.getSettings())) {
                SalaryProjectHitLog hitLog = new SalaryProjectHitLog();
                hitLog.setSalaryPoolId(salaryPoolId);
                hitLog.setSalaryProjectId(groupProjectVO.getSalaryProjectId());
                hitLog.setSalaryGroupProjectSettinId(groupProjectVO.getSettings().get(0).getId());
                hitLog.setSalaryValue(new BigDecimal(20));
                hitLog.setDataDate(localDate);
                hitLog.setGroupId(groupId);
                hitLogs.add(hitLog);
            }
        }

        for (SalaryTypeEnum salaryTypeEnum : SalaryTypeEnum.values()) {
            if (SalaryTypeEnum.ROYALTIES.equals(salaryTypeEnum) || SalaryTypeEnum.PROBATION.equals(salaryTypeEnum)) {
                continue;
            }
            SalaryPoolDetail poolDetail = new SalaryPoolDetail();
            poolDetail.setSalaryPoolId(salaryPoolId);
            poolDetail.setSalaryProjectId(null);
            poolDetail.setSalaryProjectName(salaryTypeEnum.getName());
            poolDetail.setType(salaryTypeEnum);
            poolDetail.setSalaryAmount(new BigDecimal(1000));
            poolDetail.setActualSalaryAmount(new BigDecimal(1000));
            if (SalaryTypeEnum.NECESSARY.equals(salaryTypeEnum)) {
                poolDetail.setProcessedAmount(new BigDecimal(1000));
            } else if (SalaryTypeEnum.AWARD.equals(salaryTypeEnum)) {
                poolDetail.setProcessedAmount(new BigDecimal(200));
            } else if (SalaryTypeEnum.PENALTY.equals(salaryTypeEnum)) {
                poolDetail.setProcessedAmount(new BigDecimal(100));
            } else if (SalaryTypeEnum.SUBSIDY.equals(salaryTypeEnum)) {
                poolDetail.setProcessedAmount(new BigDecimal(0));
            } else if (SalaryTypeEnum.PERSON_TAX.equals(salaryTypeEnum)) {
                poolDetail.setProcessedAmount(new BigDecimal(0));
            }
            poolDetail.setActualSalaryAmount(poolDetail.getSalaryAmount().subtract(poolDetail.getProcessedAmount()));
            poolDetail.setSalaryDate(localDate);
            poolDetail.setGroupId(groupId);
            salaryPoolDetails.add(poolDetail);
            if (salaryTypeEnum.isPlus()) {
                wages = wages.add(poolDetail.getActualSalaryAmount());
            } else {
                wages = wages.subtract(poolDetail.getActualSalaryAmount());
            }
        }
        salaryPoolDetailService.saveBatch(salaryPoolDetails);
        salaryProjectHitLogService.saveBatch(hitLogs);

        salaryPool.setReward(wages);
        salaryPoolService.saveOrUpdate(salaryPool);
        return success();
    }

}