package cn.fw.morax.service.biz;

import cn.fw.approval.sdk.param.InitiateApprovalParam;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiStarSpecialRule;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.enums.ApprovalStatusEnum;
import cn.fw.morax.domain.enums.ApprovalTypeEnum;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.rpc.approval.dto.FlowDto;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.morax.service.biz.salary.SalaryGeneralSettinBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@Slf4j
@RequiredArgsConstructor
@Service
public class ApprovalBizService {

    private final SalaryGeneralSettinBizService salaryGeneralSettinBizService;
    private final ApprovalRecordService approvalRecordService;
    private final FlowApproveRpc flowApproveRpc;
    private final EhrRpcService ehrRpcService;
    private final FlowVal flowVal;

    /**
     * 提交审批
     * @param salaryGroup
     */
    @Transactional(rollbackFor = Exception.class)
    public void applyApproveSalaryGroup(SalaryGroup salaryGroup, LoginAuthBean user, Integer staffNum){
        Date expTime = PublicUtil.getExpTimeOfMonth(salaryGroup.getBeginTime());
        //提交审批
        List<InitiateApprovalParam.Condition> conditions = Lists.newArrayList();
        InitiateApprovalParam.Condition postCondition = new InitiateApprovalParam.Condition();
        postCondition.setParamName("postId");
        postCondition.setValue(salaryGroup.getPostId().toString());
        InitiateApprovalParam.Condition shopCondition = new InitiateApprovalParam.Condition();
        shopCondition.setParamName("shopId");
        List<String> shopIdsStr = salaryGroup.getShopIds().stream().map(s -> s.toString()).collect(Collectors.toList());
        shopCondition.setValue(String.join(",", shopIdsStr));
        conditions.add(postCondition);
        conditions.add(shopCondition);
        Map extDataMap = new HashMap<String, Object>() {{
            put("id", salaryGroup.getId().toString());
        }};
        FlowDto flowDto = FlowDto.builder()
                .groupId(salaryGroup.getGroupId())
                .approvalNo(flowVal.getSalaryGroupConfig())
                .briefContent(Lists.newArrayList(
                        "申请人:" + user.getUserName(),
                        "岗位:" + salaryGroup.getPostName(),
                        "绩效项目:" + salaryGroup.getProjectNum() + "个",
                        "适用门店:" + salaryGroup.getShopIds().size() + "个",
                        "适用人员:" + staffNum + "人",
                        "申请时间:" + DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm").format(LocalDateTime.now())
                ))
                .initiatorId(user.getUserId())
                .expTime(expTime)
                .shopIds(salaryGroup.getShopIds())
                .conditions(conditions)
                .extData(extDataMap)
                .build();
        String approvalNo = flowApproveRpc.initiate(flowDto);
        ApprovalRecord approvalRecord = ApprovalRecord.builder()
                .approvalNo(approvalNo)
                .approvalStatus(ApprovalStatusEnum.IN_APPROVAL)
                .approvalType(ApprovalTypeEnum.SALARY)
                .dataId(salaryGroup.getId())
                .groupId(salaryGroup.getGroupId())
                .build();
        approvalRecordService.save(approvalRecord);
    }

    /**
     * 提交审批
     * @param kpiGroup
     */
    @Transactional(rollbackFor = Exception.class)
    public void applyApproveKpi(KpiGroup kpiGroup, LoginAuthBean user, Integer staffNum){
        Date expTime = PublicUtil.getExpTimeOfMonth(kpiGroup.getBeginTime());
        //提交审批
        List<InitiateApprovalParam.Condition> conditions = Lists.newArrayList();
        InitiateApprovalParam.Condition postCondition = new InitiateApprovalParam.Condition();
        postCondition.setParamName("postId");
        postCondition.setValue(kpiGroup.getPostId().toString());
        InitiateApprovalParam.Condition shopCondition = new InitiateApprovalParam.Condition();
        shopCondition.setParamName("shopId");
        List<String> shopIdsStr = kpiGroup.getShopIds().stream().map(s -> s.toString()).collect(Collectors.toList());
        shopCondition.setValue(String.join(",", shopIdsStr));
        conditions.add(postCondition);
        conditions.add(shopCondition);
        Map extDataMap = new HashMap<String, Object>() {{
            put("id", kpiGroup.getId().toString());
        }};
        FlowDto flowDto = FlowDto.builder()
                .groupId(kpiGroup.getGroupId())
                .approvalNo(flowVal.getKpiGroupConfig())
                .briefContent(Lists.newArrayList(
                        "申请人:" + user.getUserName(),
                        "岗位:" + kpiGroup.getPostName(),
                        "绩效项目:" + kpiGroup.getIndicatorNum() + "个",
                        "适用门店:" + kpiGroup.getShopIds().size() + "个",
                        "适用人员:" + staffNum + "人",
                        "申请时间:" + DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm").format(LocalDateTime.now())
                ))
                .initiatorId(user.getUserId())
                .expTime(expTime)
                .shopIds(kpiGroup.getShopIds())
                .conditions(conditions)
                .extData(extDataMap)
                .build();
        String approvalNo = flowApproveRpc.initiate(flowDto);
        ApprovalRecord approvalRecord = ApprovalRecord.builder()
                .approvalNo(approvalNo)
                .approvalStatus(ApprovalStatusEnum.IN_APPROVAL)
                .approvalType(ApprovalTypeEnum.KPI)
                .dataId(kpiGroup.getId())
                .groupId(kpiGroup.getGroupId())
                .build();
        approvalRecordService.save(approvalRecord);
    }

    /**
     * 提交审批
     * @param specialRule
     */
    @Transactional(rollbackFor = Exception.class)
    public void applyKpiStarAdjustment(KpiStarSpecialRule specialRule, LoginAuthBean user){
        Date expTime = salaryGeneralSettinBizService.getSalaryAppealExpTime(specialRule.getGroupId());
        //员工基础信息
        StaffBaseInfoDTO staffInfo = ehrRpcService.queryStaffBaseInfo(specialRule.getUserId());
        //提交审批
        List<InitiateApprovalParam.Condition> conditions = Lists.newArrayList();
        InitiateApprovalParam.Condition postCondition = new InitiateApprovalParam.Condition();
        postCondition.setParamName("postId");
        postCondition.setValue(staffInfo.getPostId().toString());
        InitiateApprovalParam.Condition shopCondition = new InitiateApprovalParam.Condition();
        shopCondition.setParamName("shopId");
        shopCondition.setValue(staffInfo.getShopId().toString());
        conditions.add(postCondition);
        conditions.add(shopCondition);
        Map extDataMap = new HashMap<String, Object>() {{
            put("id", specialRule.getId().toString());
        }};
        FlowDto flowDto = FlowDto.builder()
                .groupId(specialRule.getGroupId())
                .approvalNo(flowVal.getKpiStarAdjustment())
                .briefContent(Lists.newArrayList(
                        "申请人:" + user.getUserName(),
                        "岗位:" + staffInfo.getPostName(),
                        "在职门店:" + staffInfo.getShopName(),
//                        "调整方式:" + staffInfo.getShopName(),
                        "调整原因:" + specialRule.getReason(),
                        "申请时间:" + DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm").format(LocalDateTime.now())
                ))
                .initiatorId(user.getUserId())
                .expTime(expTime)
                .shopIds(Collections.singletonList(staffInfo.getShopId()))
                .conditions(conditions)
                .extData(extDataMap)
                .build();
        String approvalNo = flowApproveRpc.initiate(flowDto);
        ApprovalRecord approvalRecord = ApprovalRecord.builder()
                .approvalNo(approvalNo)
                .approvalStatus(ApprovalStatusEnum.IN_APPROVAL)
                .approvalType(ApprovalTypeEnum.COMPLAIN)
                .dataId(specialRule.getId())
                .groupId(specialRule.getGroupId())
                .build();
        approvalRecordService.save(approvalRecord);
    }

    /**
     * 审批
     * @param approvalNo
     * @param agree
     */
    @Transactional(rollbackFor = Exception.class)
    public ApprovalRecord closeApprove(String approvalNo, Boolean agree){
        ApprovalRecord approvalRecord = approvalRecordService.getOne(Wrappers.<ApprovalRecord>lambdaQuery()
                .eq(ApprovalRecord::getApprovalNo, approvalNo));
        //记录不存在、记录已被修改返回空，不再继续执行
        if (PublicUtil.isEmpty(approvalRecord)) {
            return null;
        }
        if (PublicUtil.isNotEmpty(approvalRecord) && (! ApprovalStatusEnum.IN_APPROVAL.equals(approvalRecord.getApprovalStatus()))) {
            return null;
        }
        approvalRecordService.update(Wrappers.<ApprovalRecord>lambdaUpdate()
                .set(ApprovalRecord::getApprovalStatus, (agree) ? ApprovalStatusEnum.PASS : ApprovalStatusEnum.REFUSE)
                .eq(ApprovalRecord::getApprovalNo, approvalNo)
        );
        return approvalRecord;
    }


}
