package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.salary.SalaryGroupProject;
import cn.fw.morax.domain.db.salary.SalaryGroupProjectSettin;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.enums.SalaryValueTypeEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : ConditionSalaryCalculator
 * @description : 条件薪资计算器
 * @date : 2022-04-28 14:34
 */
@Component
@Slf4j
public class ConditionSalaryCalculator extends SalaryBaseCalculator {
    @Override
    public BigDecimal calculate(SalaryGroupProject param1, SalaryGroupUser param2) {
        List<SalaryGroupProjectSettin> settins = queryProjectSettins(param1.getId());
        if (CollectionUtils.isEmpty(settins)) {
            return BigDecimal.ZERO;
        }
        Optional<JSONObject> projectValue = queryRawDataValue(param2.getUserId(), param2.getDataDate(), param1.getSalaryProjectCode(), SalaryValueTypeEnum.CONDITION);
        if (! projectValue.isPresent()) {
            return BigDecimal.ZERO;
        }
        List<BigDecimal> royalties = new ArrayList<>(settins.size());
        for (SalaryGroupProjectSettin settin : settins) {
            Long key = settin.getStairKey();
            BigDecimal stairValue = Optional.ofNullable(settin.getStairValue()).orElse(BigDecimal.ZERO);
            BigDecimal conditionValue = Optional.ofNullable(projectValue.get().getBigDecimal(String.valueOf(key))).orElse(BigDecimal.ZERO);
            BigDecimal coditionMoeny = stairValue.multiply(conditionValue);
            saveHitLog(param1.getSalaryProjectId(), settin.getId(), conditionValue, param2);
            royalties.add(coditionMoeny);
        }
        return royalties.stream().reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.CONDITION;
    }
}
