package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.salary.SalaryGroupProject;
import cn.fw.morax.domain.db.salary.SalaryGroupProjectSettin;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.db.salary.SalaryRawData;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.enums.SalaryValueTypeEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;
import java.util.Optional;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : FixattionSalaryCalculator
 * @description : 固定金额计算器
 * @date : 2022-04-27 10:34
 */
@Component
@Slf4j
public class FixattionSalaryCalculator extends SalaryBaseCalculator {
    @Override
    public BigDecimal calculate(SalaryGroupProject param1, SalaryGroupUser param2) {
        if (Boolean.TRUE.equals(param2.getProbationer())) {
            return null;
        }
        SalaryGroupProjectSettin projectSettin = queryProjectSettins(param1.getId()).stream().findAny().orElse(null);
        if (Objects.isNull(projectSettin)) {
            return BigDecimal.ZERO;
        }
        saveHitLog(param1.getSalaryProjectId(), projectSettin.getId(), BigDecimal.ZERO, param2);
        BigDecimal ratio = calcWorkDayProportion(param2);
        if (getWorkAgeCode().equalsIgnoreCase(param1.getSalaryProjectCode())) {
            Optional<SalaryRawData> rawData = queryRawData(param2.getUserId(), param2.getDataDate(), param1.getSalaryProjectCode(), SalaryValueTypeEnum.CONDITION);
            Optional<JSONObject> projectValue = checkRawDataAndGetDataValue(rawData, param2.getUserId(), param1.getSalaryProjectCode());
            BigDecimal conditionValue = Optional.ofNullable(projectValue.get().getBigDecimal(getWorkAgeCode())).orElse(BigDecimal.ZERO);
            return projectSettin.getStairValue().multiply(ratio).multiply(conditionValue).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
        }
        return projectSettin.getStairValue().multiply(ratio).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.FIXATION;
    }
}
