package cn.fw.morax.service.biz.kpi;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.ApprovalResultEvent;
import cn.fw.morax.common.pojo.event.KpiGroupChangeEvent;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.EventBusUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.dto.*;
import cn.fw.morax.domain.dto.query.KpiGroupQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupRepeatQueryDTO;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.service.biz.ApprovalBizService;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class KpiGroupBizService {

    private final KpiGroupIndicatorLaddersService kpiGroupIndicatorLaddersService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiStarLaddersService kpiStarLaddersService;
    private final KpiGroupDataService kpiGroupDataService;
    private final ApprovalBizService approvalBizService;
    private final IndicatorsService indicatorsService;
    private final KpiGroupService kpiGroupService;

    @Value("${spring.cache.custom.global-prefix}:kpi:group:save:")
    @Getter
    private String savePrefix;

    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<KpiGroupVO> kpiGroupPage(KpiGroupQueryDTO dto) {
        PageData<KpiGroup> pageData = kpiGroupService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<KpiGroup>lambdaQuery()
                        .eq(KpiGroup::getGroupId, dto.getGroupId())
                        .eq(PublicUtil.isNotEmpty(dto.getPostId()), KpiGroup::getPostId, dto.getPostId())
                        .eq(KpiGroup::getYn, Boolean.TRUE)
                        .last("ORDER BY FIELD(`status`,3,1,2,4)")
        );
        return PublicUtil.toPage(pageData, kpiGroup -> {
            KpiGroupVO kpiGroupVo = PublicUtil.copy(kpiGroup, KpiGroupVO.class);
            kpiGroupVo.setBeginTime(YearMonth.from(kpiGroup.getBeginTime()));
            return kpiGroupVo;
        });
    }

    /**
     * 绩效组详情
     * @param kpiGroupId
     * @return
     */
    public KpiGroupVO kpiGroupDetail(Long kpiGroupId) {
        KpiGroup kpiGroup = kpiGroupService.getById(kpiGroupId);
        BV.notNull(kpiGroup, "绩效配置不存在，请重试");
        KpiGroupVO kpiGroupVo = KpiGroupVO.convertKpiGroup(kpiGroup);
        List<KpiStarLadders> starLadders = kpiStarLaddersService.list(Wrappers.<KpiStarLadders>lambdaQuery()
                .eq(KpiStarLadders::getKpiGroupId, kpiGroupId)
                .eq(KpiStarLadders::getYn, Boolean.TRUE)
        );
        List<KpiStarLaddersVO> starLaddersVos = PublicUtil.copyList(starLadders, KpiStarLaddersVO.class);
        for (KpiStarLaddersVO starLaddersVO : starLaddersVos) {
            starLaddersVO.setUpper(starLaddersVO.getUpper().multiply(Constant.ONE_HUNDRED));
            starLaddersVO.setLower(starLaddersVO.getLower().multiply(Constant.ONE_HUNDRED));
        }
        kpiGroupVo.setStarLadders(starLaddersVos);

        List<KpiGroupIndicator> kpiGroupIndicators = kpiGroupIndicatorService.list(Wrappers.<KpiGroupIndicator>lambdaQuery()
                .eq(KpiGroupIndicator::getKpiGroupId, kpiGroupId)
                .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
        );
        List<KpiGroupIndicatorVO> kpiGroupIndicatorVOS = PublicUtil.copyList(kpiGroupIndicators, KpiGroupIndicatorVO.class);
        for (KpiGroupIndicatorVO indicatorVO : kpiGroupIndicatorVOS) {
            if (KpiTargetTypeEnum.RATIO.equals(indicatorVO.getTargetType())) {
                indicatorVO.setTargetValue(indicatorVO.getTargetValue().multiply(Constant.ONE_HUNDRED));
            }
        }
        List<Long> kpiGroupIndicatorIds = kpiGroupIndicatorVOS.stream().map(KpiGroupIndicatorVO::getId).collect(Collectors.toList());
        if (PublicUtil.isEmpty(kpiGroupIndicatorIds)) {
            kpiGroupVo.setIndicators(kpiGroupIndicatorVOS);
            return kpiGroupVo;
        }
        List<KpiGroupIndicatorLadders> kpiGroupIndicatorLadders = kpiGroupIndicatorLaddersService.list(Wrappers.<KpiGroupIndicatorLadders>lambdaQuery()
                .in(KpiGroupIndicatorLadders::getKpiGroupIndicatorId, kpiGroupIndicatorIds)
                .eq(KpiGroupIndicatorLadders::getYn, Boolean.TRUE)
                .orderByAsc(KpiGroupIndicatorLadders::getId)
        );
        Map<Long, List<KpiGroupIndicatorLadders>> indicatorLaddersMap = kpiGroupIndicatorLadders.stream().collect(Collectors.groupingBy(KpiGroupIndicatorLadders::getKpiGroupIndicatorId));
        for (KpiGroupIndicatorVO indicatorVo : kpiGroupIndicatorVOS) {
            if (indicatorLaddersMap.containsKey(indicatorVo.getId())) {
                List<KpiGroupIndicatorLaddersVO> laddersVos = PublicUtil.copyList(indicatorLaddersMap.get(indicatorVo.getId()), KpiGroupIndicatorLaddersVO.class);
                for (KpiGroupIndicatorLaddersVO laddersVO : laddersVos) {
                    laddersVO.convertToPercent();
                }
                indicatorVo.setIndicatorLadders(laddersVos);
            }
        }
        kpiGroupVo.setIndicators(kpiGroupIndicatorVOS);
        return kpiGroupVo;
    }

    /**
     * 绩效组保存
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getPostId()", message = "保存中，请勿重复操作")
    public KpiGroupSavePromptVO saveKpi(KpiGroupDTO dto, LoginAuthBean user) {
        //初始化数据
        this.initKpiDate(dto);
        //检查数据
        this.checkKpi(dto);
        this.checkKpiLadders(dto.getIndicators());
        this.checkStarLadders(dto.getStarLadders(), dto.getStarEvaluationType());
        KpiGroupSavePromptVO promptVo = this.checkShopKpiRepeat(dto);
        if (promptVo.getPrompt()) {
            return promptVo;
        }
        //保存相关数据
        KpiGroup kpiGroup = kpiGroupDataService.saveKpiGroup(dto);
        kpiGroupDataService.saveKpiIndicators(dto.getIndicators(), kpiGroup);
        kpiGroupDataService.saveKpiStarLadders(dto.getStarLadders(), kpiGroup);
        Integer staffNum = kpiGroupUserBizService.queryShopPostCurStaffNum(dto.getPostId(), dto.getShopIds());
        approvalBizService.applyApproveKpi(kpiGroup, user, staffNum);
        return new KpiGroupSavePromptVO(false);
    }

    /**
     * 检查绩效组配置状态
     *
     * @param dto
     */
    public void checkKpi(KpiGroupDTO dto) {
        checkNameRepetition(dto.getId(), dto.getName());
        Long postId = dto.getPostId();
        List<Long> shopIds = dto.getShopIds();
        List<KpiGroup> effectKpis = kpiGroupService.getApprovingKpis(postId, shopIds, dto.getId(), dto.getBeginTime());
        if (PublicUtil.isNotEmpty(effectKpis)) {
            KpiGroup kpiGroup = effectKpis.get(0);
            List<String> shopNames = new ArrayList<>(kpiGroup.getShopNames());
            shopNames.retainAll(dto.getShopNames());
            throw new BusinessException("绩效组门店【" + String.join(",", shopNames) + "】存在" +
                    kpiGroup.getStatus().getName() + "配置");
        }

        Set<Long> indicatorIds = new HashSet<>();
        dto.getIndicators().stream().forEach(indicator -> {
            if (indicatorIds.contains(indicator.getIndicatorId())) {
                throw new BusinessException("【" + indicator.getIndicatorName() + "】指标已配置，不能重复配置");
            }
            indicatorIds.add(indicator.getIndicatorId());
        });

        List<Indicators> indicators = indicatorsService.list(Wrappers.<Indicators>lambdaQuery()
                .in(Indicators::getId, indicatorIds)
                .eq(Indicators::getYn, Boolean.TRUE)
        );
        Map<Long, KpiTargetTypeEnum> indicatorTargetMap = indicators.stream().collect(
                Collectors.toMap(Indicators::getId, Indicators::getTargetType, (v1, v2) -> v1));
        dto.getIndicators().stream().forEach(indicator -> {
            if (! indicatorTargetMap.containsKey(indicator.getIndicatorId())) {
                throw new BusinessException("【" + indicator.getIndicatorName() + "】指标不存在或被禁用，请重试");
            }
            if (! KpiTargetTypeEnum.NO.equals(indicatorTargetMap.get(indicator.getIndicatorId()))) {
                if (PublicUtil.isEmpty(indicator.getTargetValue())) {
                    throw new BusinessException("【" + indicator.getIndicatorName() + "】指标目标值未设置，请重试");
                }
            }
        });

        //原来的绩效组配置不能在审批中
        if (PublicUtil.isEmpty(dto.getId())) {
            return;
        }
        KpiGroup kpiGroup = kpiGroupService.getById(dto.getId());
        BV.notNull(kpiGroup, "绩效配置不存在，请重试");
        BV.isFalse((SettingStatusEnum.APPROVING.equals(kpiGroup.getStatus())), "审批中的绩效组配置不能编辑");
        dto.setKgc(kpiGroup.getKgc());
    }

    /**
     * 检查绩效组排名名称是否重复
     *
     * @param id
     * @param name
     */
    public void checkNameRepetition(Long id, String name) {
        int count = kpiGroupService.count(Wrappers.<KpiGroup>lambdaQuery()
                .eq(KpiGroup::getName, name)
                .ne(PublicUtil.isNotEmpty(id), KpiGroup::getId, id)
                .eq(KpiGroup::getYn, Boolean.TRUE)
        );
        BV.isTrue(count <= 0, "绩效组名称重复，请重新输入");
    }

    /**
     * 检查指标修改的权限
     * @param dto
     */
    private void initKpiDate(KpiGroupDTO dto) {
        LocalDate currentTime = LocalDate.now();
        dto.setBeginTime((EffectMonthEnum.CURRENT_MONTH.equals(dto.getBeginTimeType()) ? currentTime : PublicUtil.getNextMonthFirstDay()));
    }

    /**
     * 检查绩效组配置是否重复
     * 同月
     * 之前没有这些门店配置，不提示
     * 有这些门店配置
     *      1)门店和之前相同不提示
     *      2)门店比之前少，之前ABC门店，现A门店，提示BC门店缺失绩效组配置
     *      3)门店比之前多，配置1 ABC门店, 配置2 DE门店,新增ABCD门店，提示E门店缺失绩效配置
     * @param kpiGroupDTO
     */
    private KpiGroupSavePromptVO checkShopKpiRepeat(KpiGroupDTO kpiGroupDTO) {
        //强制提交，退出检查
        if (kpiGroupDTO.getForce()) {
            return new KpiGroupSavePromptVO(false);
        }

        YearMonth yearMonth = YearMonth.now();
        YearMonth beginTime = YearMonth.from(kpiGroupDTO.getBeginTime());
        StringBuilder msgBuilder = new StringBuilder();
        if (yearMonth.equals(beginTime)) {
            KpiGroupRepeatQueryDTO effectRepeatDTO = KpiGroupRepeatQueryDTO.convertDto(kpiGroupDTO, SettingStatusEnum.EFFECTIVE);
            List<KpiGroup> effectKpiGroups = kpiGroupService.queryRepeatKpiGroup(effectRepeatDTO);
            if (PublicUtil.isNotEmpty(effectKpiGroups)) {
                msgBuilder.append("审批通过后").append(yearMonth.getMonth().getValue()).append("月份已生效绩效组配置会被移除，门店【");
                HashSet<String> shopNames = effectKpiGroups.stream().map(KpiGroup::getShopNames).collect(HashSet::new, HashSet::addAll, HashSet::addAll);
                msgBuilder.append(String.join(",", shopNames));
                msgBuilder.append("】。");
            }
        }
        KpiGroupRepeatQueryDTO beEffectRepeatDTO = KpiGroupRepeatQueryDTO.convertDto(kpiGroupDTO, SettingStatusEnum.BE_EFFECTIVE);
        List<KpiGroup> beKpiGroups = kpiGroupService.queryRepeatKpiGroup(beEffectRepeatDTO);
        if (PublicUtil.isNotEmpty(beKpiGroups)) {
            if (msgBuilder.length() == 0) {
                msgBuilder.append("审批通过后");
            }
            Map<LocalDate, List<KpiGroup>> monthKpiMap = beKpiGroups.stream().collect(Collectors.groupingBy(KpiGroup::getBeginTime));
            for (Map.Entry<LocalDate, List<KpiGroup>> monthKpi : monthKpiMap.entrySet()) {
                msgBuilder.append(monthKpi.getKey().getMonth().getValue()).append("月份待生效绩效组配置会被移除，门店【");
                HashSet<String> shopNames = monthKpi.getValue().stream().map(KpiGroup::getShopNames).collect(HashSet::new, HashSet::addAll, HashSet::addAll);
                msgBuilder.append(String.join(",", shopNames));
            }
            msgBuilder.append("】。");
        }
        if (msgBuilder.length() == 0) {
            return new KpiGroupSavePromptVO(false);
        }
        return new KpiGroupSavePromptVO(true, msgBuilder.toString());
    }

    /**
     * 检查阶梯分值
     * @param indicators
     */
    private void checkKpiLadders(List<KpiGroupIndicatorDTO> indicators) {
        Integer baseScore = null;
        List<KpiGroupIndicatorLaddersDTO> indicatorLadders = null;
        for (KpiGroupIndicatorDTO indicatorDto : indicators) {
            if (ScoreWayEnum.NORMAL.equals(indicatorDto.getScoreWay())) {
                continue;
            }
            baseScore = indicatorDto.getBaseScore();
            indicatorLadders = indicatorDto.getIndicatorLadders();
            Collections.sort(indicatorLadders);

            //阶梯值校验  标准分不能超过指标基础分，阶梯值必须连续
            for (KpiGroupIndicatorLaddersDTO laddersDtos : indicatorLadders) {
                //阶梯标准分不能超过绩效分值
                if (laddersDtos.getStandardScore().compareTo(baseScore) > 0) {
                    throw new BusinessException("【" + indicatorDto.getIndicatorName() + "】的阶梯标准分不能超过绩效分值");
                }
            }
            //校验阶梯
            CommonService.checkLadders(indicatorLadders, indicatorDto.getIndicatorName());
        }
    }

    /**
     * 检查星级阶梯分值
     * 1. 赋初始值
     * 2. 校验连续性、上限下限大小、所有星级都已经配置
     * @param laddersDTOS
     */
    private void checkStarLadders(List<KpiStarLaddersDTO> laddersDTOS, StarEvaluationEnum starEvaluationType) {
        Set<StarLevelEnum> levelEnumSet = Arrays.stream(StarLevelEnum.values()).collect(Collectors.toSet());
        Collections.sort(laddersDTOS, new Comparator<KpiStarLaddersDTO>() {
            @Override
            public int compare(KpiStarLaddersDTO o1, KpiStarLaddersDTO o2) {
                if (starEvaluationType.equals(StarEvaluationEnum.SCORING_RATE)) {
                    return o2.getLevel().getValue() - o1.getLevel().getValue();
                } else {
                    return o1.getLevel().getValue() - o2.getLevel().getValue();
                }
            }
        });
        //校验阶梯
        CommonService.checkLadders(laddersDTOS);
        //阶梯值校验  标准分不能超过指标基础分，阶梯值必须连续
        for (KpiStarLaddersDTO dto : laddersDTOS) {
            if (! levelEnumSet.remove(dto.getLevel())) {
                throw new BusinessException("【" + dto.getLevel().getName() + "】级的阶梯配置重复");
            }
        }
        if (levelEnumSet.size() != 0) {
            List<String> levels = levelEnumSet.stream().map(level -> {
                return level.getName() + "级";
            }).collect(Collectors.toList());
            throw new BusinessException("【" + String.join(",", levels) + "】阶梯未配置");
        }
    }

    /**
     * 审批kpiGroup
     * 生效时间 在 当前时间 之前，那么这个审批是无效的
     * 次月生效
     * 1. 将次月生效的重复绩效配置设置为失效，等待月定时器执行
     * 当月生效
     * 1. 将当月生效的重复绩效配置设置为失效，将之前配置设置为无效，将状态设置为生效中
     * @param approvalRecord
     * @param result
     */
    @Transactional(rollbackFor = Exception.class)
    public void approvalKpiGroup(ApprovalRecord approvalRecord, ApprovalResultEvent result) {
        log.info("收到绩效组配置变更审批信息：{}", JSON.toJSONString(approvalRecord));
        KpiGroup kpiGroup = kpiGroupService.getById(approvalRecord.getDataId());
        if (! result.getAgree()) {
            kpiGroupDataService.modifyStatusById(kpiGroup, SettingStatusEnum.INEFFECTIVE);
            return;
        }

        //通知绩效组人员
        KpiGroupChangeEvent kpiGroupChangeEvent = PublicUtil.copy(kpiGroup, KpiGroupChangeEvent.class);
        EventBusUtil.asyncPost(kpiGroupChangeEvent);

        YearMonth yearMonth = YearMonth.now();
        YearMonth beginTime = YearMonth.from(kpiGroup.getBeginTime());
        kpiGroup.getBeginTime();
        if (yearMonth.isAfter(beginTime)) {
            log.info("审批通过时间在绩效组生效时间之后，等待定时器执行");
            return;
        }

        //处理重复绩效组配置数据
        this.processRepeatKpiGroup(kpiGroup);
    }

    /**
     * 处理重复数据
     * 1. 次月之后生效，需要处理次月后重复的待生效数据
     * 2. 当月生效，   需要处理当月后重复的待生效数据，处理当前重复的生效数据
     * @param kpiGroup
     */
    public void processRepeatKpiGroup(KpiGroup kpiGroup){
        YearMonth yearMonth = YearMonth.now();
        YearMonth beginTime = YearMonth.from(kpiGroup.getBeginTime());
        //删除重复的待生效配置数据（时间在此绩效组配置之后、同岗位、同门店）
        KpiGroupRepeatQueryDTO beEffectRepeatDTO = KpiGroupRepeatQueryDTO.convertToBeEffectiveDTO(kpiGroup);
        List<KpiGroup> beKpiGroups = kpiGroupService.queryRepeatKpiGroup(beEffectRepeatDTO);
        kpiGroupDataService.delInEffective(beKpiGroups);

        SettingStatusEnum status = SettingStatusEnum.BE_EFFECTIVE;
        //当月生效，处理当前重复的生效数据
        if (yearMonth.equals(beginTime)) {
            status = SettingStatusEnum.EFFECTIVE;
            KpiGroupRepeatQueryDTO effectRepeatDTO = KpiGroupRepeatQueryDTO.convertToEffectiveDTO(kpiGroup);
            List<KpiGroup> effectKpiGroups = kpiGroupService.queryRepeatKpiGroup(effectRepeatDTO);
            kpiGroupDataService.modifyStatusByKpis(effectKpiGroups, SettingStatusEnum.INEFFECTIVE);
        }

        kpiGroupDataService.modifyStatusById(kpiGroup, status);
    }

    /**
     * 所有集团生效的绩效组配置
     */
    public Map<Long, List<KpiGroup>> postEffectKpis(){
        Date queryDate = DateUtil.localDateTime2Date(LocalDateTime.now().minusDays(1L));
        List<KpiGroup> kpiGroups = kpiGroupService.getAllEffectGroups(queryDate);
        if (PublicUtil.isEmpty(kpiGroups)) {
            log.info("时间：{}，没有正在生效中的绩效组配置", new SimpleDateFormat("yyyy-MM-dd").format(queryDate));
            return Maps.newHashMap();
        }
        return kpiGroups.stream().collect(Collectors.groupingBy(KpiGroup::getPostId));
    }

}
