package cn.fw.morax.service.biz.kpi;

import cn.fw.common.web.annotation.DisLock;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiStarRule;
import cn.fw.morax.domain.dto.KpiStarRuleDTO;
import cn.fw.morax.domain.vo.kpi.KpiStarRuleVO;
import cn.fw.morax.service.data.kpi.KpiStarRuleService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class KpiStarRuleBizService {

    private final KpiStarRuleService kpiStarRuleService;

    @Value("${spring.cache.custom.global-prefix}:kpi:star:rule:save:")
    @Getter
    private String savePrefix;

    /**
     * 获取绩效星级配置
     *
     * @param groupId
     * @return
     */
    public KpiStarRuleVO getKpiStarConfigByGroup(Long groupId) {
        KpiStarRule kpiStarRule = kpiStarRuleService.getOne(Wrappers.<KpiStarRule>lambdaQuery()
                .eq(KpiStarRule::getGroupId, groupId));
        if (PublicUtil.isEmpty(kpiStarRule)) {
            return new KpiStarRuleVO();
        }
        return PublicUtil.copy(kpiStarRule, KpiStarRuleVO.class);
    }

    /**
     * 绩效星级配置保存
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getGroupId()", message = "保存中，请勿重复操作")
    public void saveKpiStarConfig(KpiStarRuleDTO dto){
        kpiStarRuleService.remove(Wrappers.<KpiStarRule>lambdaQuery()
                .eq(KpiStarRule::getGroupId, dto.getGroupId()));

        KpiStarRule newKpiStarRule = PublicUtil.copy(dto, KpiStarRule.class);
        kpiStarRuleService.save(newKpiStarRule);
    }

}
