package cn.fw.morax.service.component;

import cn.fw.morax.common.pojo.event.KpiStarAdjustmentEvent;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.service.biz.salary.SalaryCalcService;
import cn.fw.morax.service.data.salary.SalaryGroupUserService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.eventbus.Subscribe;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.time.YearMonth;
import java.util.List;
import java.util.Objects;

@Component
@Slf4j
public class KpiStarAdjustmentObserver {
    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryPoolService salaryPoolService;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:salary-change:salary-recalculate")
    @Getter
    private String retrySalaryCalcKey;

    @Autowired
    public KpiStarAdjustmentObserver(final SalaryGroupUserService salaryGroupUserService,
                                     final SalaryPoolService salaryPoolService,
                                     final StringRedisTemplate stringRedisTemplate) {
        this.salaryPoolService = salaryPoolService;
        this.salaryGroupUserService = salaryGroupUserService;
        this.stringRedisTemplate = stringRedisTemplate;
    }

    /**
     * 重新计算用户工资
     *
     * @param event
     */
    @Subscribe
    public void reCalcSalary(KpiStarAdjustmentEvent event) {
        log.info("绩效等级变动重新计算用户工资[{}]", event);
        salaryRecalculatePrepare(event.getUserId(), event.getYearMonth());
    }


    /**
     * 绩效变动导致的用户薪酬重新计算准备
     *
     * @param userId
     * @param monthly
     */
    private void salaryRecalculatePrepare(Long userId, YearMonth monthly) {
        List<SalaryPool> list = salaryPoolService.list(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getUserId, userId)
                .eq(SalaryPool::getMonthly, monthly)
                .eq(SalaryPool::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        final String recalculateKey = getRetrySalaryCalcKey();
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(recalculateKey);
        for (SalaryPool pool : list) {
            SalaryGroupUser groupUser = salaryGroupUserService.getOne(Wrappers.<SalaryGroupUser>lambdaQuery()
                    .eq(SalaryGroupUser::getUserId, pool.getUserId())
                    .eq(SalaryGroupUser::getPostId, pool.getPostId())
                    .eq(SalaryGroupUser::getShopId, pool.getShopId())
                    .eq(SalaryGroupUser::getYn, Boolean.TRUE)
                    .orderByDesc(SalaryGroupUser::getDataDate)
                    .last(" limit 1 "));
            if (Objects.nonNull(groupUser)) {
                setOps.add(String.valueOf(groupUser.getId()));
            }
        }
    }
}