package cn.fw.morax.service.data.kpi;

import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.dto.query.KpiGroupRepeatQueryDTO;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author kurisu
 */
public interface KpiGroupService extends IService<KpiGroup> {
    /**
     * 根据指标编码和岗位码查询集团绩效组指标项
     *
     * @param indicatorCode 指标编码
     * @param postId        岗位id
     * @param groupId       集团id
     * @param dataDate      数据日期
     * @return
     */
    List<Long> queryKgiGroupIdByIndCode(final String indicatorCode, final Long postId, final Long groupId, final Date dataDate);

    /**
     * 根据指标编码和岗位码查询集团绩效组指标项
     *
     * @param indicatorCode 指标编码
     * @param postId        岗位id
     * @param groupId       集团id
     * @param dataDate      数据日期
     * @return
     */
    List<KpiGroup> queryKgiGroupByIndCode(final String indicatorCode, final Long postId, final Long groupId, final Date dataDate);

    /**
     * 查询重复的绩效组配置，以集团、门店、岗位维度
     * 1. 当月重复数据
     * 1）查询已生效数据   条件：状态
     * 2）查询未生效数据   条件：状态、时间（时间>=其他配置生效时间）
     * 2. 次月之后重复数据
     * 1）查询次月之后未生效数据   条件：状态、时间（时间>=其他配置生效时间）
     *
     * @param dto
     */
    List<KpiGroup> queryRepeatKpiGroup(KpiGroupRepeatQueryDTO dto);


    /**
     * 查询指定日期生效中的的绩效组id集合
     *
     * @param localDate 日期
     * @return
     */
    @Nullable
    Set<Long> queryCalculableKpiGroupIds(@NonNull final LocalDate localDate);

    /**
     * 通过id获取，包括已删除数据
     *
     * @param id
     * @return
     */
    KpiGroup getAllDataById(Long id);

    /**
     * 获取生效中数据
     *
     * @return
     */
    KpiGroup getEffectGroup(Long shopId, Long postId, Date date);

    /**
     * 获取所有生效中绩效组数据
     *
     * @return
     */
    List<KpiGroup> getAllEffectGroups(Date date);

    /**
     * 门店岗位重复的绩效组
     *
     * @param postId
     * @param shopIds
     * @param preId
     * @return
     */
    List<KpiGroup> getApprovingKpis(Long postId, List<Long> shopIds, Long preId, LocalDate beginTime);

    /**
     * 查询指定日期生效中的的绩效组集合
     *
     * @param localDate 日期
     * @return
     */
    List<KpiGroup> queryKgiGroupsByDay(LocalDate localDate);
}
